#include "OgreColorCubeMapRenderTechnique.h"
#include "OgreTechniqueGroup.h"
#include "OgreColorCubeMapRenderingRun.h"

OgreColorCubeMapRenderTechnique::OgreColorCubeMapRenderTechnique(unsigned long startFrame,
														unsigned long cubeMapUpdateInterval,
														unsigned int cubeMapResolution,												
														unsigned char texID,
														bool useDistCalc,
														bool useFaceAngleCalc,
														float distTolerance,
														float angleTolerance,
														bool updateAllFace,
														bool renderSelf,
														Pass* pass,
														OgreRenderable* parentRenderable,
														OgreTechniqueGroup* parentTechniqueGroup)
							:OgreCubeMapRenderTechnique(startFrame, cubeMapUpdateInterval, cubeMapResolution, texID, useDistCalc, useFaceAngleCalc, distTolerance, angleTolerance, updateAllFace, renderSelf, pass, parentRenderable, parentTechniqueGroup),
							ColorCubeMapRenderTechnique(startFrame, cubeMapUpdateInterval, cubeMapResolution, useDistCalc, useFaceAngleCalc, distTolerance, angleTolerance, updateAllFace, renderSelf, parentRenderable, parentTechniqueGroup),
							CubeMapRenderTechnique(startFrame, cubeMapUpdateInterval, cubeMapResolution, useDistCalc, useFaceAngleCalc, distTolerance, angleTolerance, updateAllFace, renderSelf, parentRenderable, parentTechniqueGroup),
							RenderTechnique(parentRenderable, parentTechniqueGroup) 
{	
	
	if(sharedRuns->getRun(ILLUMRUN_COLOR_CUBEMAP) == 0)
		sharedRuns->addRun(ILLUMRUN_COLOR_CUBEMAP, createColorCubeMapRun());
	
	colorCubeMapRunChanged(sharedRuns->getRun(ILLUMRUN_COLOR_CUBEMAP));
}


OgreColorCubeMapRenderTechnique::~OgreColorCubeMapRenderTechnique()
{

}

void OgreColorCubeMapRenderTechnique::update(unsigned long frameNum)
{
	ColorCubeMapRenderTechnique::update(frameNum);
	/*
	GpuProgramParametersSharedPtr fpParams = pass->getFragmentProgramParameters();
	Vector3 center = ((OgreSharedRuns*) sharedRuns)->getRootPosition();
	fpParams->setNamedConstant("lastCenter",center); 
	pass->setFragmentProgramParameters(fpParams); */
	
}

void OgreColorCubeMapRenderTechnique::colorCubeMapRunChanged(RenderingRun* run)
{
	OgreColorCubeMapRenderingRun* cuberun =(OgreColorCubeMapRenderingRun*) (run->asOgreRenderingRun());
	String cubemapname = cuberun->getColorCubeMapTextureName();
	
	pass->getTextureUnitState(texID)->setTextureName(cubemapname);
}

RenderingRun* OgreColorCubeMapRenderTechnique::createColorCubeMapRun()
{
	return new OgreColorCubeMapRenderingRun( (OgreSharedRuns*) parentTechniqueGroup->getSharedRuns(), 
												parentOgreRenderable->getName()  + "_COLORCUBEMAP",
												startFrame,
												cubeMapUpdateInterval,
												cubeMapResolution,
												useDistCalc,
												useFaceAngleCalc,
												distTolerance,
												angleTolerance,
												updateAllFace);
}


///Technique factory
OgreColorCubeMapRenderTechniqueFactory::OgreColorCubeMapRenderTechniqueFactory()
{
	typeName = "ColorCubeMap";	
}

OgreRenderTechnique* OgreColorCubeMapRenderTechniqueFactory::createInstance( 
										IllumTechniqueParams* params,
										Pass* pass,
										OgreRenderable* parentRenderable,
										OgreTechniqueGroup* parentTechniqueGroup)
{	
	
	OgreCubeMapRenderTechniqueFactory::resetParams();
	OgreCubeMapRenderTechniqueFactory::parseParams(params);	
	parseParams(params);
	
	OgreColorCubeMapRenderTechnique* result = new OgreColorCubeMapRenderTechnique(
													startFrame,
													cubeMapUpdateInterval,
													cubeMapResolution,
													texID,
													useDistCalc,
													useFaceAngleCalc,
													distTolerance,
													angleTolerance,
													updateAllFace,
													renderSelf,
													pass,
													parentRenderable,
													parentTechniqueGroup);
	
	return result;
}

